;*======================================================== *
;*                     SCRSAVE.ASM                         *
;*          (C) 1993 Heinz Klaus & DMV-Verlag              *
;*               Assembler: TASM 2.X, 3.X                  *
;* ------------------------------------------------------- *
;* Schaltet den VGA-Bildschirm nach Ablauf der eingestell- *
;* ten Zeit (1 bis 60 Minuten) ab. Durch Drcken einer     *
;* Taste oder einer Mausbewegung wird der Bildschirm       *
;* wieder aktiviert. Die Deinstallation des TSR-Programms  *
;* erfolgt durch einen erneuten Aufruf.                    *
;* ScrSave stellt sich automatisch auf die in der          *
;* config.sys eingestellte Landessprache ein (deutsch,   *
;* franzsisch oder englisch)                              *
;* ------------------------------------------------------- *
;* Assemblieren: TASM scrsave  |  TLINK scrsave /t         *
;* ======================================================= *
.8086
.MODEL TINY

LOCALS @@                   ;* Labelkollisionen vermeiden

;*======================================================== *
;*                       Makros:                           *

PushAll   MACRO             ;* Register auf den Stack retten
            PUSH AX
            PUSH BX
            PUSH CX
            PUSH DX
            PUSH DI
            PUSH SI
            PUSH ES
            PUSH DS
            PUSH CS         ;* und Datensegment
            POP  DS         ;* restaurieren
          ENDM

PopAll    MACRO             ;* Register wieder vom Stack 
            POP  DS         ;* holen
            POP  ES
            POP  SI
            POP  DI
            POP  DX
            POP  CX
            POP  BX
            POP  AX
          ENDM

;*======================================================== *
;*                      Konstanten                         *

id  = 6B68H                 ;* Programmkennung 'hk' (6Bh=k /
                            ;* 68h=h) fr die Deinstallation

;*======================================================== *
;*                        Code                             *

CODE      SEGMENT  PARA 'CODE'

          ORG 00H           ;* erste Speicherstelle im PSP. 
ident     DW  ?             ;* Hier wird die Kennung 'hk' 
                            ;* gespeichert
          ORG 2CH                 
EnvSeg    DW  ?             ;* Segm.-Adresse des Environment

          ORG 100H          ;* Startadresse des Programms

ASSUME CS: CODE, DS: CODE, ES: CODE, SS: CODE

Start:    JMP Init          ;* Sprung zur Installations-
                            ;* routine

;*======================================================== *
;*               Daten (verbleiben im Speicher)            *

Int8_Old  EQU THIS DWORD    ;* Adresse des alten Timer-Intr.
Int8_Ofs  DW  ?             ;* Offsetadresse
Int8_Seg  DW  ?             ;* Segmentadresse

Int9_Old  EQU THIS DWORD    ;* Adr. des alten Tastatur-Intr.
Int9_Ofs  DW  ?             ;* Offsetadresse
Int9_Seg  DW  ?             ;* Segmentadresse

TTime     DW  0             ;* Zhler fr Abschaltzeitpunkt
TCount    DW  0             ;* Abschalten nach
                            ;*   Minuten * 60 * 18 Ticks
Mouse     DB  0             ;* Maustreiber installiert?
                            ;*   (1 = JA)

;*======================================================== *
;*           Routine fr den Tastaturinterrupt             * 

ASSUME CS: CODE, DS: CODE, ES: NOTHING, SS: NOTHING

Int9_New  PROC FAR
          STI               ;* Interrupt wieder zulassen
          PushAll           ;* Register sichern
          CMP  TCount, 0    ;* Bildschirm schon aus?
          JNZ  @@1          ;* wenn NEIN, dann weiter ...

          ;* == Einschalt-Routine fr VGA-Karten ==

          MOV  AX, 1200H    ;* wenn JA, dann den Bildschirm
          MOV  BL, 36H      ;* via Interrupt 10h wieder ein-
          INT  10H          ;* schalten (AL=00h, AH=12h)
         
@@1:      MOV  AX, TTime    ;* vorgegebene Zeiteinheiten 
          MOV  TCount, AX   ;* nach AX und Zhler neu setzen
          PopAll            ;* Register wieder restaurieren

ASSUME CS: CODE, DS: NOTHING, ES: NOTHING, SS: NOTHING

          JMP  Int9_Old     ;* zum alten Interrupt-Handler
                            ;* springen

Int9_New  ENDP

;*======================================================== *
;*             Routine fr den Timerinterrupt              *

ASSUME CS: CODE, DS: CODE, ES: NOTHING, SS: NOTHING

Int8_New  PROC FAR
          STI               ;* Interrupts wieder zulassen
          PushAll           ;* Alle Register sichern
          CMP  Mouse, 0     ;* Ist ein Maustreiber aktiv?
          JZ   Timer        ;* Wenn NEIN, dann weiter ...
          JA   M_Check      ;* Wenn JA, ab zur Mausroutine

          ;* eigentl. Timer-Interrupt fr die Zeitzhlung

Timer:    CMP  TCount, 0    ;* Bildschirm schon aus?
          JZ   P_Exit       ;* Wenn JA, dann weiter....
          DEC  TCount       ;* Wenn NEIN, Zhlvariable -1
          JNZ  P_Exit       ;* Wenn tcount > 0, dann weiter

         ;* Abschalt-Routine fr VGA-Karten

          MOV  AX, 1201H    ;* wenn tcount = 0, dann den 
          MOV  BL, 36H      ;* VGA-Bildschirm via Int 10h
          INT  10H          ;* Funkt.12h / Uf. 01 abschalten

P_Exit:   PopAll            ;* Register wieder restaurieren

ASSUME CS: CODE, DS: NOTHING, ES: NOTHING, SS: NOTHING

          JMP  Int8_Old     ;* zum alten Interrupt-Handler
                            ;* springen
Int8_New  ENDP

Mouse_N   EQU THIS BYTE     ;* ab hier wird der Speicher 
                            ;* freigegeben (im Zuge der
                            ;* Installation) wenn kein
                            ;* Maustreiber installiert ist

;* Daten (nur fr die Mausabfrage-Routine)

MTime     DW  9             ;* Zhler fr Aufruf des Int 33h
MCount    DW  9             ;* Aufruf nach 9 Ticks = 2x/sec
XPos      DW  0             ;* Mausposition horizontal
YPos      DW  0             ;* Mausposition vertikal

;*======================================================== *
;*              Routine fr die Mausabfrage                *
M_Check   PROC NEAR
          DEC  MCount       ;* Zeitzhler fr Mausabfrage -1
          JNZ  Timer        ;* wenn noch nicht 0, dann
                            ;* weiter ...

          ;* Mauszustand berprfen (zweimal pro Sekunde)

          MOV  AX, MTime    ;* Zeitzhler fr Mausabfrage 
          MOV  MCount, AX   ;* wieder auf Anfangswert setzen
          MOV  AX, 0003H    ;* Maus-Interrupt 33h (Funkt. 03
          INT  33H          ;* Unterfunkt. 0h) aufrufen

          CMP  XPos, CX     ;* hat sich eine der Maus
          JNE  @@1          ;* positionen (XY) verndert?
          CMP  DX, YPos     ;* wenn JA, Bildschirm wieder 
          JE   Timer        ;* einschalten (nur wenn
                            ;* abgeschaltet!)
          ;* bei Mausbewegung wieder Bildschirm aktivieren
@@1:      MOV  XPos, CX     ;* momentane Mausposition merken
          MOV  YPos, DX     ;* (Position XY)
          CMP  TCount, 0    ;* Bildschirm schon aus?
          JNZ  @@2          ;* wenn NEIN, dann weiter ...

          ;* Einschalt-Routine fr VGA-Karten

          MOV  AX, 1200H    ;* wenn JA, dann den Bildschirm
          MOV  BL, 36H      ;* via Interrupt 10h (Funkt. 12h
          INT  10H          ;* Uf. 00h) wieder einschalten

@@2:      MOV  AX, TTime    ;* Zeit (wann soll Abschaltung 
          MOV  TCount, AX   ;* aktiviert werden) nach AX 
          JMP  SHORT P_Exit ;* und Zhler setzen. Danach
                            ;* Sprung zur Timer-Routine
M_Check   ENDP

Mouse_Y   EQU THIS BYTE     ;* ab hier wird der Speicher 
                            ;* freigegeben (im Zuge der 
                            ;* Installation) wenn ein
                            ;* Maustreiber installiert und
                            ;* aktiv ist

;*======================================================== *
;*               Installations-Routine                     *

ASSUME CS: CODE, DS: CODE, ES: CODE, SS: CODE

Init      PROC NEAR
          CALL Country      ;* Landessprache einstellen
          MOV  AX, 3508H    ;* Inhalt des Interruptvektors 
          INT  21H          ;* via DOS-Funktion 35 holen
                            ;* (Int 08h)
          CMP  ES:ident, id ;* Identittskennung vorhanden?
          JE  SHORT DeInst  ;* wenn JA, Programm deinstal-
                            ;* lieren

          ;* Programm installieren

          MOV  Int8_Seg, ES ;* Seg/Ofs-Adresse des aktuellen
          MOV  Int8_Ofs, BX ;* Interrupts speichern

          CALL P_Check      ;* VGA, Maus, und Parameter
                            ;* berprfen..

          MOV  AX, 3509H    ;* Inhalt des Interruptvektors 
          INT  21H          ;* via DOS-Funktion 35 holen
                            ;* (Int 09h)
          MOV  Int9_Seg, ES ;* Seg/Ofs-Adresse des aktuellen
          MOV  Int9_Ofs, BX ;* Interrupt speichern
          MOV  ident, id    ;* Identittskennung (hk) ab-
                            ;* speichern
          LEA  DX, Int8_New ;* Ofs/Adr. der neuen Interrupt-
          MOV  AX, 2508H    ;* Routine (08h) holen und
                            ;* Interruptvektor auf die 
          INT  21H          ;* eigene Routine verbiegen
          LEA  DX, Int9_New ;* Ofs/Adr. der neuen Interrupt-
          MOV  AX, 2509H    ;* Routine (09) holen und Inter-
          INT  21H          ;* ruptvektor auf die eigene
                            ;* Routine verbiegen
          MOV  DX, Msg1     ;* Text-Zeiger von  
          MOV  AH, 9        ;* '......(c) hk 1991' laden und
          INT  21H          ;* via DOS ausgeben
          MOV  DX, Msg2     ;* Text-Zeiger von 'installiert'
          INT  21H          ;*  laden und ausgeben

          ;* nicht bentigten Speicher freigeben

          LEA  DX,Mouse_N   ;* Anzahl der Paragraphen (ohne
          CMP  Mouse, 0     ;* Maus), (je 16 Byte) ...
          JZ   @@1          ;* keine Maus aktiv, dann weiter
          LEA  DX, Mouse_Y  ;* Anzahl der Paragraphen (mit
@@1:      MOV  CL, 4        ;* Maus), (je 16 Byte).... ber
          SHR  DX, CL       ;* die das residente Programm 
          INC  DX           ;* verfgen soll, berechnen.
          MOV  AX, 3100H    ;* Programm via DOS beenden, 
          INT  21H          ;* aber resident halten
Init      ENDP

;*======================================================== *
;*                Programm deinstallieren                  *

DeInst    PROC NEAR
          LDS  DX, ES:Int8_Old ;* Inhalt des Int-Vektors auf
          MOV  AX, 2508H    ;* die alte Routine zurcklegen
          INT  21H
          LDS  DX, ES:Int9_Old ;* Inhalt des Int-Vektors auf 
          MOV  AX, 2509H    ;* die alte Routine zurcklegen
          INT  21H
          MOV  BX, ES       ;* SegAdr. des installierten
                            ;* Programms merken
          MOV  ES, ES:EnvSeg;* SegAdr. des Environments aus
                            ;* dem PSP holen ...
          MOV  AH, 49H      ;* und den Speicher des alten
          INT  21H          ;* Environments wieder freigeben
          MOV  ES, BX       ;* ebenso den Speicher des 
          MOV  AH, 49H      ;* installierten Programms 
          INT  21H          ;* wieder freigeben
          PUSH CS           ;* CS auf den Stack legen und 
          POP  DS           ;* als DS wieder holen
          MOV  DX, Msg1     ;* Text-Zeiger von 
          MOV  AH, 9        ;* '......(c) hk 1991' laden 
          INT  21H          ;* und via DOS ausgeben
          MOV  DX, Msg3     ;* Text-Zeiger von 
          INT  21H          ;* 'deinstalliert' laden und 
          MOV  AX, 4C00H    ;* ausgeben und ...
          INT  21H          ;* Programm beenden
DeInst    ENDP

;*======================================================== *
;*          System- und Parameterberprfung               *

P_Check   PROC NEAR
          ;* Ist eine VGA-Grafikkarte installiert und aktiv?
          MOV  AX, 1A00H    ;* mit Interrupt 10h (Funkt. 
          INT  10H          ;* 1Ah) auf VGA-Karte prfen
          CMP  AL, 1AH      ;* ist aktive Karte eine VGA?
          JE   @@1          ;* wenn JA, dann weiter...
          MOV  DX, Msg5     ;* Text-Zeiger von 
          JMP  SHORT Msg    ;* '.....keine VGA...' laden und
                            ;* zur Textausgabe springen...

          ;* Ist ein Maustreiber installiert und aktiv?

@@1:      XOR  AX, AX       ;* berprfen ob ein Maustreiber
          INT  33H          ;* installiert und bereit ist
          CMP  AX, 0        ;* AX > 0  = Treiber installiert
          JZ   @@2          ;* wenn NEIN, dann weiter....
          INC  Mouse        ;* Flag fr Abfrage auf 'TRUE'
                            ;* setzen
          
          ;* Kommandozeilen-Parameter bearbeiten

@@2:      MOV  SI, 80H      ;* PSP-Adresse: Anzahl Parameter
          CMP  BYTE PTR [SI], 0 ;* Parameter vorhanden?
          JZ   @@6          ;* wenn NEIN, Fehler!....
          MOV  BH, 10       ;* fr die Rechnung im Dez-Syst.
          XOR  AL, AL       ;* bisheriges Ergebnis ist Null
          MOV  AH, 1        ;* Flag fr: noch keine Zahl
                            ;* gefunden
@@3:      INC  SI           ;* PSP-Adresse erhhen u. Zchen.
          MOV  BL, BYTE PTR [SI] ;* aus Kommandozeile holen

          CMP  BL, 0DH      ;* ist Zeichen ein CR?
          JE   @@5          ;* wenn JA, Abfrage beendet
          CMP  BL, 20H      ;* ist Zeichen ein Leerzeichen?
          JNE  @@4          ;* wenn NEIN, weiter springen
          CMP  AH, 0        ;* wenn JA, prfen, war letztes
                            ;* Zeichen ein Zahl?
          JZ   @@6          ;* wenn JA, Fehler
          JA   @@3          ;* wenn NEIN, weiter 
@@4:      CMP  BL, '0'      ;* ist Zeichen eine Ziffer?
          JB   @@6          ;* wenn NEIN, Fehler !
          CMP  BL, '9'      ;* ist Zeichen eine Ziffer?
          JA   @@6          ;* wenn NEIN, Fehler !

         ;* ASCII nach binr umwandeln
          
          MUL  BH           ;* bisher errechnete Zahl * 10
          AND  BL, 15       ;* Ziffer nach binr umwandeln
          ADD  AL, BL       ;* und zum bisherigen Ergebnis
                            ;* addieren
          CMP  AL, 60       ;* Zahl grer 60?
          JA   @@6          ;* wenn JA, Fehler....
          JBE  @@3          ;* wenn NEIN, weiter in der
                            ;* Runde....
@@5:      CMP  AL, 0        ;* ist das Ergebnis Null?
          JZ   @@6          ;* wenn JA, Fehler....

         ;* Zeitumrechnung fr den Zeitzhler

          MOV  BX, 60       ;* Parameterangabe (1..60 Min)
          MUL  BX           ;* * 60 Sekunden und * 18 Ticks
          MOV  BX, 18       ;* pro Sekunde ergibt den Zhler 
          MUL  BX           ;* fr die Zeitrechnung
          MOV  TTime, AX    ;* Ergebnis in den Vars TTIME
          MOV  TCount, AX   ;* und TCOUNT festhalten
          RET
@@6:      MOV  DX, Msg6     ;* Text-Zeiger von
                            ;* 'Aufruf: SCRSAVE...' laden
          JMP  SHORT Msg    ;* und zur Textausgabe springen
P_Check   ENDP

;*======================================================== *
;*                Textausgabe/Fehler                       *

Msg       PROC NEAR
          PUSH DX           ;* Fehlermeldung auf Stack
                            ;* retten
          MOV  AH, 9        ;* Funktionsnummer fr
                            ;* Stringausgabe
          MOV  DX, Msg1     ;* Text-Zeiger von
                            ;* '.....(c) hk 1991' laden und
          INT  21H          ;* und am Bildschirm ausgeben
          MOV  DX, Msg4     ;* Text-Zeiger von
                            ;* 'FEHLER !.....' laden und am
          INT  21H          ;* Bildschirm ausgeben
          POP  DX           ;* Fehlerart (Text) vom Stack
          INT  21H          ;* holen und ausgeben
          MOV  DX, Msg7     ;* Text-Zeiger von 'Programm ..'
          INT  21H          ;* laden und ausgeben
          XOR  AX, AX       ;* BIOS-Aufruf 'warte ...
          INT  16H          ;* 
          MOV  DX, Msg8     ;* Text-Zeiger von 10,10,13,'$''
          MOV  AH, 9        ;* laden und ausgeben
          INT  21H
          MOV  AX, 4C00H    ;* Programm via DOS beenden
          INT  21H          ;* (nicht resident !)
Msg       ENDP

;*======================================================== *
;*          Text-Labels und der Textpuffer                 *

Msg1    DW  0       ;* Label des Textes 1  (Zeiger auf die)
Msg2    DW  0       ;* Label des Textes 2  (Adresse inner-)
Msg3    DW  0       ;* Label des Textes 3  (halb des Text-)
Msg4    DW  0       ;* Label des Textes 4  (puffers.      )
Msg5    DW  0       ;* Label des Textes 5  ( < msg_buf >  )
Msg6    DW  0       ;* Label des Textes 6
Msg7    DW  0       ;* Label des Textes 7
Msg8    DW  0       ;* Label des Textes 8

Strg_E  DB '$'      ;* Endezeichen fr DOS-Strings

Msg_Buf DB  320 DUP (?)   ;* Puffer fr den jeweiligen Text-
                          ;* teil, deutsch, englisch, franz.

Cou_Buf DB   34 DUP (?)   ;* Puffer fr die landesspezif.
                          ;* Daten  (hier nur Dummy !)


;*======================================================== *
;*        Ausgabetexte in den einzelnen Sprachen           *

CRLF   EQU 13, 10
   
German DB CRLF, 'ScrSave 1.01 (c) Heinz Klaus & DMV 1993 $'
       DB 'installiert',   CRLF, '$'
       DB 'deinstalliert', CRLF, '$'
       DB CRLF, 'Fehler!', CRLF, '$'
       DB 'Es ist keine VGA-Karte aktiv', CRLF, '$'
       DB 'Richtiger Aufruf: scrsave <1..60 Minuten>', CRLF
       DB '$'
       DB 'Das Programm wurde nicht installiert!', CRLF,CRLF
       DB 'Bitte drcken Sie eine Taste ', 7, '$'
       DB CRLF, '$'
Len_G  EQU $ - German
      ;* Lnge des deutschen Textblocks berechnen...

French DB CRLF, 'ScrSave 1.01 (c) Heinz Klaus & DMV 1993 $'
       DB 'install',        CRLF, '$'
       DB 'deinstall',      CRLF, '$'
       DB 13, 10, 'Erreur!', CRLF, '$'
       DB "Il n'est pas de carte VGA active", CRLF, '$'
       DB 'Correct appel: scrsave <1..60 minutes>', CRLF,'$'
       DB "Le programme n'installera pas!", CRLF, CRLF
       DB 'Frappez une touche ', 7, '$'
       DB CRLF, '$'
Len_F  EQU $ - French
      ;* Lnge des franzsischen Textblocks berechnen.

Englsh DB CRLF, 'ScrSave 1.01 (c) Heinz Klaus & DMV 1993 $'
       DB 'installed', CRLF, '$'
       DB 'deinstalled', CRLF, '$'
       DB 13, 10, 'Error!', CRLF, '$'
       DB 'Here is no VGA card active', CRLF, '$'
       DB 'Correct command: scrsave <1..60 minutes>'
       DB CRLF, '$'
       DB 'The program will not be installed!', CRLF, CRLF
       DB 'Press any key to continue ', 7, '$'
       DB CRLF, '$'
Len_E  EQU $ - Englsh
      ;* Lnge des englischen Textblocks berechnen...

;*======================================================== *
;* Initialisierungsroutine fr die jeweilige Landessprache *

Country   PROC NEAR
          MOV  AX, CS       ;* Segmentadresse nach DS und ES
          MOV  DS, AX       ;* laden, wegen der Str.-Befehle
          MOV  ES, AX       ;* LODSB und STOSB

          ;* aktuellen Landescode via DOS (ab 3.0)
          ;* Interrupt holen

          LEA  DX, Cou_Buf  ;* Pufferadr. f. die landes-
                            ;* spezifischen Daten
          MOV  AX, 3800H    ;* via DOS Interrupt 21h/
                            ;* Funktion 3800h
          INT  21H          ;* die aktuelle Lndereinstel-
                            ;* lung holen; in BX steht der
                            ;* aktuelle Lndercode!

          ;* und entsprechende Textadresse + Textblocklnge
          ;* laden

@@0:      LEA  SI, German   ;* Adr. des deutschen Textblocks
          MOV  CX, Len_G    ;* und dessen Gesamtlnge laden
          CMP  BX, 49       ;* Lndercode Deutschland?
          JE   @@1          ;* wenn JA, dann zur Kopier-
                            ;* Routine....
          CMP  BX, 43       ;* Lndercode sterreich?
          JE   @@1          ;* wenn JA, dann zur Kopier-
                            ;* Routine....
          CMP  BX, 41       ;* Lndercode Schweiz?
          JE   @@1          ;* wenn JA, dann zur Kopier-
                            ;* Routine....

          LEA  SI, French   ;* Adresse des franzsischen
                            ;* Textblocks
          MOV  CX, Len_F    ;* und dessen Gesamtlnge laden
          CMP  BX, 33       ;* Lndercode Frankreich?
          JE   @@1          ;* wenn JA, dann zur Kopier-
                            ;* Routine....

          ;* hier knnten weitere Landescode abgefragt
          ;* werden!

          LEA  SI, Englsh   ;* englische oder unbekannte
                            ;* Landesangabe,
          MOV  CX, Len_E    ;* daher englische Sprache!

@@1:      LEA  DI, Msg_Buf  ;* Textpufferadresse laden und
                            ;* sie dem ersten Textlabel
          MOV  [Msg1], DI   ;* bergeben

          ;* Kopier-Routine/verschiebt die Landestexte
          ;* in den Puffer

          XOR  BX, BX       ;* BX auf Null setzen/wegen
                            ;* Adressierung
          CLD               ;* Directionflag lschen

@@2:      LODSB             ;* erstes Zeichen der Landes-
                            ;* sprache holen und im
          STOSB             ;* Textpuffer ablegen
          DEC  CX           ;* Textlnge -1
          JCXZ @@3          ;* Textlnge schon 0, dann raus

          CMP  AL, Strg_E   ;* ein String-Ende Zeichen
                            ;* entdeckt?
          JNE  @@2          ;* wenn NEIN, weiter in der
                            ;* Runde....

          INC  BX           ;* Zeichen entdeckt daher die
                            ;* Adresse des nchsten
          INC  BX           ;* Textlabel berechnen und
          MOV  [Msg1+BX], DI;* Adresse (innerhalb d.Puffers)
                            ;* bergeben
          JMP  SHORT @@2    ;* weiter in der Schleife bis
                            ;* CX = 0
@@3:      RET

Country   ENDP

CODE      ENDS
          END  Start

;*======================================================== *
;*                  Ende von SCRSAVE.ASM                   *
