{$define RETAIL_VERSION}
{!define Win32}
{ ***************************************************************************
  Source File Name     :  SYSTOOLS.PAS
  Autor                :  Mario M. Westphal
  Erstellt am          :  10.06.1991

  Compiler             :  Turbo Pascal for Windows 1.5
  Betriebssystem       :  DOS 5.0, Windows 3.x
  Compiler-Schalter    :  -

  Bemerkungen          :  -

  Beschreibung         :  Allgemeine Routinen

  Revisionen           :  1.00 10.06.1991 created (MW)
                          1.10 29.12.1992 added new funtions (MW)
                               07.04.1993 revisited (MW)
 *************************************************************************** }
{$M 8192,8192}
{$A+,B-,D+,F-,G+,I+,L+,N-,R+,S+,V+,W-,X+,Q+}

{$IFDEF RETAIL_VERSION}
  {$D-,L-,S-,R-,Q-,I-}
{$ENDIF}

unit SysTools;

INTERFACE
uses
  WinTypes,
  WinProcs,
  Objects,
  OWindows,
  ODialogs,
  Strings,
  WinDOS;


{ -- Numerische Funktionen ----------------------------------------------------- }

function Min (a,b: LongInt) : LongInt;
function Max (a,b: LongInt) : LongInt;


{ -- Datum- und Zeitfunktionen ------------------------------------------------- }

function GetDateLong : LongInt;
function GetTimeLong : LongInt;

procedure ClockOn;
function  ClockOff : Real;
function  TimeStr (t: Real) : String;
procedure TicksOn;
function  TicksOff : LongInt;

{ -- Routinen fr jeden Zweck -------------------------------------------------- }

procedure DrawBitmap (DC: HDC; BitMap: HBitmap; X, Y: Integer);
procedure KeepWindowsAlive;
procedure Delay (ms: LongInt);
function  LoadStr (AStrID: Word) : PChar;

procedure _AHIncr; FAR;

{ -- Routinen zum File-IO ------------------------------------------------------ }
function _hread (FileHandle: Integer; Buffer: PChar; nBytes: LongInt) : LongInt; FAR;
function _hwrite (FileHandle: Integer; Buffer: PChar; nBytes: LongInt) : LongInt; FAR;


IMPLEMENTATION

var
	StartTime		: Real;
	TickCount		: LongInt;

{***********************************************************}
{ Routinen fr File-IO mit Blockgre > 64K                 }
{                                                           }
{ _hread  : Liet nBytes auf den globalem Heap              }
{ _hwrite : Schreibt nBytes in eine Datei                   }
{                                                           }
{***********************************************************}

function _hread (FileHandle: Integer; Buffer: PChar; nBytes: LongInt) : LongInt; EXTERNAL 'KERNEL' INDEX 349;
function _hwrite (FileHandle: Integer; Buffer: PChar; nBytes: LongInt) : LongInt; EXTERNAL 'KERNEL' INDEX 350;


{***********************************************************}
{ Numerische Funktionen                                     }
{                                                           }
{ Min     : Liefert das Minimum zweier Integer              }
{ Max     : Liefert das Maximum zweier Integer              }
{                                                           }
{***********************************************************}

function Min (a,b: LongInt) : LongInt;
begin
	if a <= b then Min := a
	else Min := b;
end; { Min }

function Max (a,b: LongInt) : LongInt;
begin
	if a >= b then Max := a
	else Max := b;
end; { Max }


{***********************************************************}
{ Datums- und Zeitfunktionen                                }
{                                                           }
{ GetDateLong   : Ermittelt das Datum und konvertiert es    }
{                 in das DOS-Format.                        }
{ GetTimeLong   : Ermittelt die Zeit und konvertiert sie    }
{                 in das DOS-Format.                        }
{ ClockOn       : Uhr starten                               }
{ ClockOff      : Uhr stoppen                               }
{ TimeStr       : Ticks -> hh:mm:ss                         }
{ TicksOn;      : Ticks merken                              }
{ TicksOff      : Ticks auslesen                            }
{                                                           }
{***********************************************************}

{-------------------------------------------------------------------------------
}
function GetDateLong : LongInt;
var y,m,d,dow : Word;
begin
	GetDate(y,m,d,dow);
	GetDateLong := ((y-1980) shl 9) + (m shl 5) + d;
end; { GetDateLong }

{-------------------------------------------------------------------------------
}
function GetTimeLong : LongInt;
var h,m,s,s100 : Word;
begin
	GetTime(h,m,s,s100);
	GetTimeLong := (h shl 11) + (m shl 5) + (s div 2);
end; { GetTimeLong }

{-------------------------------------------------------------------------------
}
procedure ClockOn;
begin
	StartTime := GetTickCount;
end;

{-------------------------------------------------------------------------------
}
function ClockOff : Real;
begin
	ClockOff := ((GetTickCount-StartTime) / 18.2);
end;

{-------------------------------------------------------------------------------
  Erzeugt aus den bergebenen Sekunden einen String der Form hh:mm:ss
}
function TimeStr (t: Real) : String;
var
	s		: Word;
	h,m	: Byte;
	s1 	: String;
	s2	: String;
begin
	s := Round(t);
	h := s div 3600; s := s mod 3600;
	m := s div 60; s := s mod 60;
	Str(h:2,s1);
	Str(m:2,s2);
	s1 := s1+':'+s2;
	Str(s:2,s2);
	s1 := s1+':'+s2;
	for h := 1 to Length(s1) do if s1[h] = ' ' then s1[h] := '0';
	TimeStr := s1;
end; { TimeStr }

{-------------------------------------------------------------------------------
  Startet die Uhr
}
procedure TicksOn;
begin
	TickCount := GetTickCount;
end; { TicksOn }

{-------------------------------------------------------------------------------
  Liefert die seit dem letzten Aufruf von TicksOn vergangene Zeit in
  Timer-Ticks.
}
function TicksOff:LongInt;
begin
	TicksOff := GetTickCount-TickCount;
end; { TicksOff }


{***********************************************************}
{ Routinen fr jede Gelegenheit                             }
{                                                           }
{***********************************************************}

procedure _AHIncr; EXTERNAL 'KERNEL';

{----------------------------------------------------------
  Zeichnet eine Bitmap an die Position x,y
}
procedure DrawBitmap(DC: HDC; BitMap: HBitmap; X, Y: Integer);
var
  MemDC : HDC;
  B     : TBitmap;
  Size  : TPoint;
  Origin: TPoint;
  hbmOld: HBitmap;

BEGIN
  MemDC := CreateCompatibleDC(DC);
  hbmOld := SelectObject(MemDC, Bitmap);
  SetMapMode(MemDC, GetMapMode(DC));
  GetObject(Bitmap, SizeOf(TBitmap), @B);
  Size.X := B.bmWidth;
  Size.Y := B.bmHeight;
  DPtoLP(DC, Size, 1);
  Origin.X := 0;
  Origin.Y := 0;
  DPtoLP(MemDC, Origin, 1);
  BitBlt(DC, X, Y, Size.X, Size.Y, MemDC, Origin.X, Origin.Y, SRCCOPY);
  SelectObject(MemDC,hbmOld);
  DeleteDC(MemDC);
end;

{----------------------------------------------------------
  Prft, ob fr eine andere Task Nachrichten vorliegen.
  Ist dies der Fall, wird diese Anwendung aktiviert.
  Ansonsten wird sofort zum Aufrufer zurckgekehrt.
}
procedure KeepWindowsAlive;
var m : TMsg;
begin
  if PeekMessage(m,0,0,0,PM_REMOVE) then
  begin
    if not Application^.ProcessAppMsg(m) then
    begin
      TranslateMessage(m);
      DispatchMessage(m);
    end;
  end;
end;

{----------------------------------------------------------
  Analog dem normalen Delay in der DOS-Version
}
procedure Delay (ms: LongInt);
var
  lTickCount : LongInt;
begin
  lTickCount := GetTickCount;
  while GetTickCount - lTickCount < MS do KeepWindowsAlive;
end; { Delay }


{-------------------------------------------------------------------------------
  Ldt den Strinf mit der ID AStrID aus der Ressource-Datei. Kann der String
  nicht gefunden werden, wird der Standardtext "<ID>?" zurckgeliefert.
}
function LoadStr(AStrID: Word) : PChar;
const
  s : array[0..255] of Char = '';
begin
  if LoadString(HInstance,AStrID,s,SizeOf(s)) <> 0 then
    LoadStr := s
  else
  begin
    Str(AStrID,s);
    StrCat(s,'?');
    LoadStr := s;
  end;
end;

end.