(* ****************************************************** *)
(*                       REDIR.PAS                        *)
(*      Umleitung der Standardausgabe in eine Datei       *)
(*           (c) 1993 Wolfhard Rinke & DMV                *)
(* ****************************************************** *)
UNIT Redir;

{$A+,B-,D+,E+,F-,G-,I+,L+,N-,O-,P-,Q+,R+,S+,T-,V+,X+,Y+}
{$M 16384,0,655360}

INTERFACE

CONST
  stdout = 1;
  stderr = 2;

  FUNCTION RedirectStdOut(FName : STRING) : WORD;
  FUNCTION RestoreStdOut : WORD;
  FUNCTION RedirectStdErr(FName : STRING) : WORD;
  FUNCTION RestoreStdErr : WORD;
  FUNCTION RedirectErrMsg(Err : WORD) : STRING;

IMPLEMENTATION

CONST
  OutFile : STRING = 'STDOUT.RED' + #0;
  ErrFile : STRING = 'STDERR.RED' + #0;

VAR
  OldOut : WORD;
  OldErr : WORD;
  Error  : WORD;

  FUNCTION RedirectStdOut(FName : STRING) : WORD;
  BEGIN
    IF FName <> '' THEN OutFile := FName + #0;
    ASM
      MOV  BX, stdout
      MOV  AH, 45h          (* stdout-Handle verdoppeln   *)
      INT  21h
      JC   @Error1

      MOV  OldOut, AX
      MOV  DX, OFFSET OutFile + 1
      MOV  CX, 0            (* Dateiattribut: normal      *)
      MOV  AH, 3Ch          (* Ausgabedatei anlegen       *)
      INT  21h
      JC   @Error1

      MOV  BX, AX           (* Handle der Standardausgabe *)
      MOV  CX, stdout       (* auf Ausgabedatei umleiten  *)
      MOV  AH, 46h
      INT  21h
      JC   @Error1

      MOV  Error, 0
      JMP  @Ok1

    @Error1:
      MOV  Error, AX
    @Ok1:
    END;
    RedirectStdOut := Error;
  END;

  FUNCTION RestoreStdOut : WORD;
  BEGIN
    ASM
      MOV  BX, oldout       (* stdout-Handle restaurieren *)
      MOV  CX, stdout
      MOV  AH, 46h
      INT  21h
      JC   @Error2

      MOV  BX, OldOut       (* Kopie von stdout schlieen *)
      MOV  AH, 3Eh
      INT  21h
      JC   @Error2

      MOV  Error, 0
      JMP  @Ok2

    @Error2:
      MOV  Error, AX
    @Ok2:
    END;
    RestoreStdOut := Error;
  END;

  FUNCTION RedirectStdErr(FName : STRING) : WORD;
  BEGIN
    IF FName <> '' THEN ErrFile := FName + #0;
    ASM
      MOV  BX, stderr
      MOV  AH, 45h          (* stderr-Handle verdoppeln   *)
      INT  21h
      JC   @Error1

      MOV  OldErr, AX       (* alten Handle sichern       *)
      MOV  DX, OFFSET ErrFile + 1
      MOV  CX, 0            (* Dateiattribut: normal      *)
      MOV  AH, 3Ch          (* Ausgabedatei anlegen       *)
      INT  21h
      JC   @Error1

      MOV  BX, AX           (* Handle des Standardfehlers *)
      MOV  CX, stderr       (* auf Ausgabedatei umleiten  *)
      MOV  AH, 46h
      INT  21h
      JC   @Error1

      MOV  Error, 0
      JMP  @Ok1

    @Error1:
      MOV  Error, AX
    @Ok1:
    END;
    RedirectStdErr := Error;
  END;

  FUNCTION RestoreStdErr : WORD;
  BEGIN
    ASM
      MOV  BX, OldErr       (* stderr-Handle restaurieren *)
      MOV  CX, stderr
      MOV  AH, 46h
      INT  21h
      JC   @Error2

      MOV  BX, OldErr       (* Kopie von stdout schlieen *)
      MOV  AH, 3Eh
      INT  21h
      JC   @Error2

      MOV  Error, 0
      JMP  @Ok2

    @Error2:
      MOV  Error, AX
    @Ok2:
    END;
    RestoreStdErr := Error;
  END;

  FUNCTION RedirectErrMsg(Err : WORD) : STRING;
  VAR
    M : STRING;
  BEGIN
    M := 'Ausgabe nicht umgeleitet: ' + #13 + 'Fehler: ';
    CASE Err OF
      02 : M := 'Datei nicht gefunden.';
      03 : M := 'Pfad nicht gefunden.';
      04 : M := 'keine weiteren Handles verfgbar.';
      05 : M := 'Zugriff abgewiesen.';
      06 : M := 'ungltiger Handle.';
      11 : M := 'ungltiges Format.';
      12 : M := 'ungltiger Zugriffscode.';
      13 : M := 'ungltige Daten.';
      15 : M := 'unerlaubter Laufwerksbezeichner.';
    ELSE
      M := 'unbekannt.';
    END;
    RedirectErrMsg := M;
  END;

END.
(* ****************************************************** *)
(*                Ende von REDIR.PAS                      *)
